const app = require('j7/app');
const utils = require('j7/utils');
const FormulaService = require('../services/factory').create('FormulaService', null);

const constant = require('../constant');

const HERO_TYPE = 1;
const WEAPON_TYPE = 2;
const CHIP_TYPE = 3;
class Damping {

    async start() {

        console.log('Damping.start');
        while (true) {
            await this.doDamping(utils.getUtcTime());

            const nowTime = utils.getUtcTime();
            const daySeconds = utils.getDaySeconds(nowTime, constant.TIME_ZONE);
            const sleepTime = daySeconds + 3600 * 24 - nowTime;
            console.log('Damping.sleepTime:' + sleepTime, new Date(), sleepTime / 60);
            await utils.sleep(sleepTime * 1000);
        }
    }

    async doDamping(nowTime) {
        try {

            const daySeconds = utils.getDaySeconds(nowTime, constant.TIME_ZONE);
            const yesterdaySeconds = daySeconds - 86400;
            console.log('today:'+daySeconds)
            console.log('yesterday:'+yesterdaySeconds)
            const {err, conn} = await app.getDbConn("GameDb20060");
            if (err) {
                throw err;
            }
            if (!err && conn) {
                const isExec = await this.isExec(conn, nowTime);
                if (! isExec){
                    const nftList = await this.yesterdayActiveNft(conn,yesterdaySeconds);  //获取昨日活跃产出ceg的nft列表
                    this.updateNftLaborValue(conn,nftList)  ;                               //对nft的劳力值进行递增
                    this.dampingTili(conn,nftList);                                         //对nft进行体力衰减
                    this.increaseServiceDays(conn,nowTime);                                        //递增服务器自然天数
                }
            }
        } catch (err) {
            console.log(err);
        }
    }

    //获取昨日活跃产出ceg的nft列表
    async yesterdayActiveNft(conn,yesterdaySeconds) {
        const {err, rows} = await conn.execQuery(
            'SELECT * FROM t_nft_active WHERE daytime = ?',
            [
                yesterdaySeconds
            ]
        );
        if (err) {
            throw err;
            return;
        }
        return rows;
    }

    //递增活跃nft的劳力值
    async updateNftLaborValue(conn,nftList){
        if (nftList.length<1){
            return
        }
        await utils.serial(nftList,
            async (item,index)=> {
                    switch (item.token_type){
                        case HERO_TYPE:{
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [['!labour',  ()=>{
                                return 'labour + 1'
                            }]]
                            conn.update('t_hero',whereList,fieldList);
                        }
                        break;
                        case WEAPON_TYPE:{
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [['!labour',  ()=>{
                                return 'labour + 1'
                            }]]
                            conn.update('t_gun',whereList,fieldList);
                        }
                        break;
                        case CHIP_TYPE:{
                            const {err, row} = await conn.execQueryOne(
                                'SELECT * FROM t_chip WHERE idx = ?',
                                [
                                    item.uniid
                                ]
                            );
                            if (row.strength == 0){
                                return;
                            }
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [['!labour',  ()=>{
                                return 'labour + 1'
                            }]]
                            conn.update('t_chip',whereList,fieldList);
                        }
                        break;
                        default:{}
                    }
        });
    }

    //体力衰减
    async dampingTili(conn,nftList){
        if (nftList.length<1){
            return
        }
        await utils.serial(nftList,
            async (item,index)=> {
                    switch (item.token_type){
                        case HERO_TYPE:{
                            const {err, row} = await conn.execQueryOne(
                                'SELECT * FROM t_hero WHERE idx = ?',
                                [
                                    item.uniid
                                ]
                            );
                            let heroPsa = FormulaService.Hero_PSA_Value(row); //英雄衰减比
                            let maxTili = FormulaService.Hero_NFT_Maximum_Physical_Strength(row);//英雄最大体力值
                            let  dampTili = maxTili * heroPsa;
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [];
                            if (heroPsa == 0){
                                 fieldList = [['hero_tili',  0]];
                            }else {
                                let fun = (dampTili)  =>{
                                    return `GREATEST(0,hero_tili - ${dampTili})`
                                }
                                 fieldList = [['!hero_tili',  fun(dampTili)]];
                            }
                            conn.update('t_hero',whereList,fieldList);
                        }
                        break;
                        case WEAPON_TYPE:{
                            const {err, row} = await conn.execQueryOne(
                                'SELECT * FROM t_gun WHERE idx = ?',
                                [
                                    item.uniid
                                ]
                            );
                            let weaponDa = FormulaService.Weapon_DA_Value(row);//枪械衰减比
                            let maxTili = FormulaService.Weapon_NFT_Maximum_Durability(row);//枪械最大耐久度
                            let  dampTili = maxTili * weaponDa;
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [];
                            if (weaponDa == 0){
                                fieldList = [['durability',  0]];
                            }else {
                                let fun = (dampTili)  =>{
                                    return `GREATEST(0,durability - ${dampTili})`
                                }
                                fieldList = [['!durability',  fun(dampTili)]];
                            }

                            conn.update('t_gun',whereList,fieldList);
                        }
                        break;
                        case CHIP_TYPE:{
                            const {err, row} = await conn.execQueryOne(
                                'SELECT * FROM t_chip WHERE idx = ?',
                                [
                                    item.uniid
                                ]
                            );
                            let chipPsa = FormulaService.Chip_PSA_Value(row)//芯片衰减比
                            let dampTili = row.strength_max * chipPsa;
                            let whereList = [['idx', item.uniid]];
                            let fieldList = [];
                            if(chipPsa == 0){
                                fieldList = [['strength',  0]];
                            }else {

                                let fun = (dampTili)  =>{
                                    return `GREATEST(0,strength - ${dampTili})`
                                }
                                fieldList = [['!strength',  fun(dampTili)]];
                            }
                            conn.update('t_chip',whereList,fieldList);
                            console.log("CHIP--------->"+ chipPsa)
                            console.log("CHIP--------->"+ dampTili)
                        }
                        break;
                        default:{}
                    }

        });
    }

    //递增服务器自然天数
    async increaseServiceDays(conn,nowTime){
        const daySeconds = utils.getDaySeconds(nowTime, constant.TIME_ZONE);
        await conn.upsert(
            't_realtime_data',
            [['name','server_days']],
            [['!value1',  ()=>{
                return 'value1 + 1'
            }],['modifytime',daySeconds]],
            [['name','server_days'],['value1',1],['createtime',nowTime],['modifytime',daySeconds]]
        );
    }

    async isExec(conn,nowTime){
        const {err, row} = await conn.execQueryOne(
            'SELECT * FROM t_realtime_data WHERE name = ?',
            [
                'server_days'
            ]
        );
        if (err) {
            throw err;
            return true;
        }
        if (row && nowTime - row.modifytime < 86400){
            return true;
        }else {
            return  false;
        }
    }

    async getChipOne(conn,unid) {
        const {err, row} = await conn.execQueryOne(
            'SELECT * FROM t_chip WHERE idx = ?',
            [
                unid
            ]
        );
        if (err) {
            throw err;
            return;
        }
        return row;
    }

}

function init() {
    (new Damping()).start();
}

exports.init = init;
